import math
import matplotlib.pyplot as plt
from .Generaldistribution import Distribution

class Gaussian(Distribution):
	
	""" Gaussian distribution class for calculating and visualizing a 
	Gaussian distribution.
	
	Attributes:
		mean (float) representing the mean value of the distribution
		stdev (float) representing the standard deviation of the 
		distributiondata_list (list of floats) a list of floats 
		extracted from the data file.
			
	"""

	def __init__(self, mu=0, sigma=1):
		
		Distribution.__init__(self, mu, sigma, 0, 0)
	
	
	def calculate_mean(self):
	
		"""Function to calculate the mean of the data set.
		
		Args: 
			None
		
		Returns: 
			float: mean of the data set
	
		"""
					
		self.mean = 1.0 * sum(self.data) / len(self.data)

		return self.mean
	

	def calculate_stdev(self, sample=True):
	
		"""Function to calculate the standard deviation of the data set.
		
		Args: 
			sample (bool): whether the data represents a sample or 
			population
		
		Returns: 
			float: standard deviation of the data set
	
		"""
	
		if sample:
			n = len(self.data) - 1
		else:
			n = len(self.data)
	
		mean = self.calculate_mean()
		sigma = 0
		for d in self.data:
			sigma += (d - mean) ** 2
		
		self.stdev = math.sqrt(sigma / n)

		return self.stdev


	def pdf(self, x):
		"""Probability density function calculator for the gaussian 
		distribution.
		
		Args:
			x (float): point for calculating the probability density 
			function
			
		
		Returns:
			float: probability density function output
		"""
		
		return (1.0 / (self.stdev * math.sqrt(2*math.pi))) * \
		       math.exp(-0.5*((x - self.mean) / self.stdev) ** 2)
		
	
	def plot_histogram_pdf(self, n_spaces=50):
	
		"""Function to plot the normalized histogram of the data and a 
		plot of the probability density function along the same range.
		
		Args:
			n_spaces (int): number of data points 
		
		Returns:
			list: x values for the pdf plot
			list: y values for the pdf plot
			
		"""
		
		mu = self.mean
		sigma = self.stdev
	
		min_range = min(self.data)
		max_range = max(self.data)
		
		# calculates the interval between x values
		interval = 1.0 * (max_range - min_range) / n_spaces
	
		x = []
		y = []
		
		# calculate the x values to visualize
		for i in range(n_spaces):
			tmp = min_range + interval*i
			x.append(tmp)
			y.append(self.pdf(tmp))
	
		# make the plots
		fig, axes = plt.subplots(2,sharex=True)
		fig.subplots_adjust(hspace=.5)

		axes[0].hist(self.data, density=True)
		axes[0].set_title('Normed Histogram of Data')
		axes[0].set_ylabel('Density')
	
		axes[1].plot(x, y)
		axes[1].set_title('Normal Distribution for \n \
		Sample Mean and Sample Standard Deviation')
		axes[1].set_ylabel('Density')
		plt.show()

		return x, y


	def __add__(self, other):
		
		"""Function to add together two Gaussian distributions.
		
		Args:
			other (Gaussian): Gaussian instance
			
		Returns:
			Gaussian: Gaussian distribution
			
		"""

		result = Gaussian()
		result.mean = self.mean + other.mean
		result.stdev = math.sqrt(self.stdev ** 2 + other.stdev ** 2)
		
		return result
		
		
	def __repr__(self):
	
		"""Function to output the characteristics of the Gaussian 
		instance.
		
		Args:
			None
		
		Returns:
			string: characteristics of the Gaussian
		
		"""
		
		return "mean {} \n standard deviation {}".\
		       format(self.mean, self.stdev)


class Distribution:
	
	def __init__(self, mu=0, sigma=1, skewness=0, kurtosis=0):
	
		""" Generic distribution class for calculating and 
		visualizing a probability distribution.
	
		Attributes:
			mean (float) representing the mean value of the 
			distribution stdev (float) representing the standard 
			deviation of the distribution data_list (list of floats) 
			a list of floats extracted from the data file.
			"""
		
		self.mean = mu
		self.stdev = sigma
		self.skew = skewness
		self.kurt = kurtosis
		self.data = []


	def read_data_file(self, file_name):
	
		"""Function to read in data from a txt file. The txt file should 
		have one number (float) per line. The numbers are stored in the 
		data attribute.
				
		Args:
			file_name (string): name of a file to read from
		
		Returns:
			None
		
		"""
			
		with open(file_name) as file:
			data_list = []
			line = file.readline()
			while line:
				data_list.append(float(line))
				line = file.readline()
		file.close()
	
		self.data = data_list
		

	def plot_histogram(self):
		"""Function to output a histogram of the instance variable data 
		using matplotlib pyplot library.
		
		Args:
			None
			
		Returns:
			None
		"""
		plt.hist(self.data)
		plt.title('Histogram of Data')
		plt.xlabel('data')
		plt.ylabel('count')


import math
import matplotlib.pyplot as plt
from .Generaldistribution import Distribution

class Binomial(Distribution):
    
    """ Binomial distribution class for calculating and visualizing a Binomial 
    distribution.
    
    Attributes:
        mean (float) representing the mean value of the distribution
        stdev (float) representing the standard deviation of the distribution
        data_list (list of floats) a list of floats to be extracted from the 
        data file.
        
        p (float) representing the probability of an event occurring
        n (int) number of trials
               
    """

    def __init__(self, prob=0.5, size=20):
                
        self.n = size
        self.p = prob
        Distribution.__init__(self, self.calculate_mean(),
                              self.calculate_stdev(), self.calculate_skew(),
                              self.calculate_kurt())
    

    def calculate_mean(self):
    
        """Function to calculate the mean from p and n.
        
        Args: 
            None
        
        Returns: 
            float: mean of the data set
    
        """

        self.mean = self.p * self.n
                
        return self.mean
    

    def calculate_stdev(self):
    
        """Function to calculate the standard deviation from p and n.
        
        Args: 
            None
        
        Returns: 
            float: standard deviation of the data set
    
        """
        
        self.stdev = math.sqrt(self.n * self.p * (1 - self.p))
        
        return self.stdev
    

    def calculate_skew(self):
    
        """Function to calculate the skewness from p and n.
        
        Args: 
            None
        
        Returns: 
            float: skewness of the data set
    
        """

        self.skew = ((1 - self.p) - self.p) / math.sqrt(self.n * self.p *
                                                        (1 - self.p))
                
        return self.skew


    def calculate_kurt(self):
    
        """Function to calculate the excess kurtosis from p and n.
        
        Args: 
            None
        
        Returns: 
            float: excess kurtosis of the data set
    
        """

        self.kurt = (1 - 6 * self.p * (1 - self.p)) / \
            (self.n * self.p * (1 - self.p))
                
        return self.kurt
    

    def replace_stats_with_data(self):
    
        """Function to calculate p and n from the data set
        
        Args: 
            None
        
        Returns: 
            float: the p value
            float: the n value
    
        """
    
        self.n = len(self.data)
        self.p = 1.0 * sum(self.data) / len(self.data)
        self.mean = self.calculate_mean()
        self.stdev = self.calculate_stdev()
        self.skew = self.calculate_skew()
        self.kurt = self.calculate_kurt()
        
        return self.p, self.n


    def pdf(self, k):
        """Probability density function calculator for the gaussian 
        distribution.
        
        Args:
            x (int): point for calculating the probability density function
            
        
        Returns:
            float: probability density function output
        """

        a = math.factorial(self.n) / (math.factorial(k) *
                                      (math.factorial(self.n - k)))
        b = (self.p ** k) * (1 - self.p) ** (self.n - k)
        
        return a * b
        
    
    def plot_pdf(self):
    
        """Function to plot the pdf of the binomial distribution.
        
        Args:
            None
        
        Returns:
            list: x values for the pdf plot
            list: y values for the pdf plot
            
        """
        
        x = []
        y = []
        
        # calculate the x values to visualize
        for i in range(self.n + 1):
            x.append(i)
            y.append(self.pdf(i))
    
        # make the plots
        plt.bar(x, y)
        plt.title('Distribution of Outcomes')
        plt.ylabel('Probability')
        plt.xlabel('Outcome')
    
        plt.show()
    
        return x, y

        
    def __add__(self, other):
        
        """Function to add together two Binomial distributions with equal p.
        
        Args:
            other (Binomial): Binomial instance
            
        Returns:
            Binomial: Binomial distribution
            
        """
        
        try:
            assert self.p == other.p, 'p values are not equal'
        except AssertionError as error:
            raise
        
        result = Binomial()
        result.n = self.n + other.n
        result.p = self.p
        result.calculate_mean()
        result.calculate_stdev()
        result.calculate_skew()
        result.calculate_kurt()
        
        return result
        
        
    def __repr__(self):
    
        """Function to output the characteristics of the Binomial instance.
        
        Args:
            None
        
        Returns:
            string: characteristics of the Binomial
        
        """
        
        return "mean {} \n standard deviation {} \n skewness {} \n\
        kurtosis {} \n p {} \n n {}".format(self.mean, self.stdev, self.skew,
                                            self.kurt, self.p, self.n)



from setuptools import setup

setup(name='commondist',
      version='0.2',
      description='Common distributions',
      packages=['commondist'],
      author = 'Jason Ho',
      author_email = 'jasonho0810@gmail.com',
      zip_safe=False)



import math
import matplotlib.pyplot as plt
from .Generaldistribution import Distribution

class Poisson(Distribution):
    
    """ Poisson distribution class for calculating and visualizing a Poisson 
    distribution.
    
    Attributes:
        mean (float) representing the mean value of the distribution
        stdev (float) representing the standard deviation of the distribution
        data_list (list of floats) a list of floats to be extracted from the 
        data file.
               
    """

    def __init__(self, lam=1):

        self.lam = lam
        Distribution.__init__(self, self.calculate_mean(),
                              self.calculate_stdev(), self.calculate_skew(),
                              self.calculate_kurt())
    
                        
    def calculate_mean(self):
    
        """Function to calculate the mean of the data set.
        
        Args: 
            None
        
        Returns: 
            float: mean of the data set
    
        """
        
        self.mean = 1.0 * self.lam

        return self.mean


    def calculate_stdev(self):
    
        """Function to calculate the standard deviation of the data set.
        
        Args: 
            None
        
        Returns: 
            float: standard deviation of the data set
    
        """
        
        self.stdev = math.sqrt(self.lam)
        
        return self.stdev
    

    def calculate_skew(self):
    
        """Function to calculate the skewness of the data set.
        
        Args: 
            None
        
        Returns: 
            float: skewness of the data set
    
        """

        self.skew = self.lam ** -0.5
                
        return self.skew


    def calculate_kurt(self):
    
        """Function to calculate the excess kurtosis of the data set.
        
        Args: 
            None
        
        Returns: 
            float: excess kurtosis of the data set
    
        """

        self.kurt = self.lam ** -1
                
        return self.kurt
    

    def replace_stats_with_data(self):
    
        """Function to calculate p and n from the data set
        
        Args: 
            None
        
        Returns: 
            float: the p value
            float: the n value
    
        """
    
        self.mean = self.calculate_mean()
        self.stdev = self.calculate_stdev()
        self.skew = self.calculate_skew()
        self.kurt = self.calculate_kurt()
        
        return self.p, self.n


    def pdf(self, x):
	    """Probability density function calculator for the lognormal 
	    distribution.
	    
	    Args:
		    x (int): point for calculating the probability density 
		    function
		    
	    
	    Returns:
		    float: probability density function output
	    """
	    
	    return ((lam ** x) * (e ** -self.lam)) / math.factorial(x)

	
    def plot_histogram_pdf(self, n_spaces=50):
    
	    """Function to plot the normalized histogram of the data and a 
	    plot of the probability density function along the same range.
	    
	    Args:
		    n_spaces (int): number of data points 
	    
	    Returns:
		    list: x values for the pdf plot
		    list: y values for the pdf plot
		    
	    """
	    
	    mu = self.mean
	    sigma = self.stdev
    
	    min_range = min(self.data)
	    max_range = max(self.data)
	    
	    # calculates the interval between x values
	    interval = 1.0 * (max_range - min_range) / n_spaces
    
	    x = []
	    y = []
	    
	    # calculate the x values to visualize
	    for i in range(n_spaces):
		    tmp = min_range + interval*i
		    x.append(tmp)
		    y.append(self.pdf(tmp))
    
	    # make the plots
	    fig, axes = plt.subplots(2,sharex=True)
	    fig.subplots_adjust(hspace=.5)

	    axes[0].hist(self.data, density=True)
	    axes[0].set_title('Normed Histogram of Data')
	    axes[0].set_ylabel('Density')
    
	    axes[1].plot(x, y)
	    axes[1].set_title('Poisson Distribution for \n \
	    Sample Mean and Sample Standard Deviation')
	    axes[1].set_ylabel('Density')
	    plt.show()

	    return x, y


    def __add__(self, other):
        
        """Function to add together two Poisson distributions.
        
        Args:
            other (Poisson): Poisson instance
            
        Returns:
            Poisson: Poisson distribution
            
        """
        
        result = Poisson()
        result.lam = self.lam + other.lam
        result.calculate_mean()
        result.calculate_stdev()
        result.calculate_skew()
        result.calculate_kurt()
        
        return result
    

    def __repr__(self):
    
        """Function to output the characteristics of the Lognormal instance.
        
        Args:
            None
        
        Returns:
            string: characteristics of the Lognormal
        
        """
        
        return "mean {} \n standard deviation {} \n skewness {} \n\
        kurtosis {}".format(self.mean, self.stdev, self.skew, self.kurt)
